/**
 * This file is part of Imagepipe.
 *
 * Copyright (c) 2017, 2018, 2019, 2020, 2022 Pawel Dube
 *
 * Imagepipe is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * Imagepipe is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Imagepipe. If not, see <http://www.gnu.org/licenses/>.
 */

package de.kaffeemitkoffein.imagepipe;

import android.graphics.*;

public class ImageFilters {

    private ImageFilters(){

    }

    public static Bitmap applyColorMatrix(Bitmap source, ColorMatrix colorMatrix){
        Bitmap target = Bitmap.createBitmap(source.getWidth(),source.getHeight(), Bitmap.Config.ARGB_8888);
        ColorMatrixColorFilter colorMatrixColorFilter = new ColorMatrixColorFilter(colorMatrix);
        Canvas targetCanvas = new Canvas(target);
        Paint paint = new Paint();
        paint.setColorFilter(colorMatrixColorFilter);
        targetCanvas.drawBitmap(source,0,0,paint);
        return target;
    }

    public static final class FilerType{
        public final static int NONE = 0;
        public final static int GREYSCALE = 1;
        public final static int SEPIA = 2;
        public final static int SEPIALIGHT = 3;
        public final static int SEPIA1960 = 4;
        public final static int SEPIACOLD = 5;
        public final static int INVERT = 6;
        public final static int MYSTIFY = 7;
        public final static int DAWN = 8;
    }

    public static Bitmap grayScaleBitmap(Bitmap source){
        if (source!=null){
            ColorMatrix colorMatrix = new ColorMatrix();
            colorMatrix.setSaturation(0); // to grayscale
            return applyColorMatrix(source,colorMatrix);
        }
        return null;
    }

    /*
                        R   G   B   A
             Matrix:    1   0   0   0   0       red
                        0   1   0   0   0       green
                        0   0   1   0   0       blue
                        0   0   0   1   0       alpha
    */

    final static float[] sepiaMatrix =     {0.500f, 0.650f, 0.150f, 0, 0,
                                            0.380f, 0.570f, 0.040f, 0, 0,
                                            0.250f, 0.500f, 0.002f, 0, 0,
                                            0.000f, 0.000f, 0.000f,1f, 0};

    final static float[] sepiaColdMatrix = {0.300f, 0.570f, 0.050f, 0, 0,
                                            0.450f, 0.700f, 0.030f, 0, 0,
                                            0.500f, 0.800f, 0.020f, 0, 0,
                                            0.000f, 0.000f, 0.000f,1f, 0};


    final static float[] sepiaLightMatrix= {1f, 0, 0    , 0, 0,
                                             0 ,1f, 0    , 0, 0,
                                             0 , 0, 0.83f, 0, 0,
                                             0,  0,     0,1f, 0};

    final static float[] sepia1960Matrix=  {0.65f, 0.3f, 0.3f, 0, 0,
                                            0 ,0.6f, 0.6f, 0, 0,
                                            0 , 0, 0.80f, 0, 0,
                                            0,  0,     0,1f, 0};

    final static float[] mystifyMatrix =   {1.000f, 0.000f, -0.50f, 0, 0,
                                            0.000f, 1.000f, -0.60f, 0, 0,
                                            0.000f, 0.000f, 1.000f, 0, 0,
                                            0.000f, 0.000f, 0.000f,1f, 0};

    final static float[] dawnMatrix =      {0.500f, 0.600f, 0.100f, 0, 0,
                                            0.000f, 1.000f, 0.000f, 0, 0,
                                            0.000f, 0.200f, 1.000f, 0, 0,
                                            0.000f, 0.000f, 0.000f,1f, 0};

    final static float[] nullMatrix =      {0.000f, 0.000f, 0.000f, 0, 0,
                                            0.000f, 0.000f, 0.000f, 0, 0,
                                            0.000f, 0.000f, 0.000f, 0, 0,
                                            0.000f, 0.000f, 0.000f,1f, 0};

    final static float[] invertColorMatrix =
                                           {-1f,0 , 0 , 0, 255f,
                                            0  ,-1f, 0, 0, 255f,
                                            0, 0, -1f,0, 255f,
                                            0, 0, 0,1f, 0};

    public static Bitmap sepiaBitmap(Bitmap source){
        if (source!=null){
            ColorMatrix colorMatrix = new ColorMatrix();
            colorMatrix.set(sepiaMatrix);
            return applyColorMatrix(source,colorMatrix);
        }
        return null;
    }

    public static Bitmap invertBitmap(Bitmap source){
        if (source!=null){
            ColorMatrix colorMatrix = new ColorMatrix();
            colorMatrix.set(invertColorMatrix);
            return applyColorMatrix(source,colorMatrix);
        }
        return null;
    }

    public static Bitmap applyColorFilter(Bitmap source, int filter){
        ColorMatrix colorMatrix = null;
        if (filter==FilerType.NONE){
            return source.copy(Bitmap.Config.ARGB_8888,true);
        }
        if (filter==FilerType.GREYSCALE){
            return grayScaleBitmap(source);
        }
        if (filter==FilerType.SEPIA){
            colorMatrix = new ColorMatrix(sepiaMatrix);
        }
        if (filter==FilerType.SEPIALIGHT){
            colorMatrix = new ColorMatrix(sepiaLightMatrix);
        }
        if (filter==FilerType.SEPIA1960){
            colorMatrix = new ColorMatrix(sepia1960Matrix);
        }
        if (filter==FilerType.SEPIACOLD){
            colorMatrix = new ColorMatrix(sepiaColdMatrix);
        }
        if (filter==FilerType.INVERT){
            colorMatrix = new ColorMatrix(invertColorMatrix);
        }
        if (filter==FilerType.MYSTIFY){
            colorMatrix = new ColorMatrix(mystifyMatrix);
        }
        if (filter==FilerType.DAWN){
            colorMatrix = new ColorMatrix(dawnMatrix);
        }
        if (colorMatrix!=null){
            return applyColorMatrix(source,colorMatrix);
        }
        return null;
    }

}
